@@ -22,7 +22,6 @@ import qualified Data.Set as S
 import Data.Store
 import Data.Store.Core (unsafeEncodeWith)
 import Data.Store.Version
-import qualified Data.Text as T
 import Language.Haskell.TH
 import Path
 import Path.IO (ensureDir)
@@ -37,14 +36,14 @@ versionedDecodeFile :: Data a => VersionConfig a -> Q Exp
 versionedDecodeFile vc = [e| versionedDecodeFileImpl $(decodeWithVersionQ vc) |]
 
 -- | Write to the given file.
-storeEncodeFile :: (Store a, MonadIO m, MonadLogger m, Eq a)
+storeEncodeFile :: (Store a, MonadIO m, MonadReader env m, HasCallStack, HasLogFunc env, Eq a)
                 => (a -> (Int, Poke ()))
                 -> Peek a
                 -> Path Abs File
                 -> a
                 -> m ()
 storeEncodeFile pokeFunc peekFunc fp x = do
-    let fpt = T.pack (toFilePath fp)
+    let fpt = fromString (toFilePath fp)
     logDebug $ "Encoding " <> fpt
     ensureDir (parent fp)
     let (sz, poker) = pokeFunc x
@@ -55,14 +54,14 @@ storeEncodeFile pokeFunc peekFunc fp x = do
 -- | Read from the given file. If the read fails, run the given action and
 -- write that back to the file. Always starts the file off with the
 -- version tag.
-versionedDecodeOrLoadImpl :: (Store a, Eq a, MonadUnliftIO m, MonadLogger m)
+versionedDecodeOrLoadImpl :: (Store a, Eq a, MonadUnliftIO m, MonadReader env m, HasCallStack, HasLogFunc env)
                           => (a -> (Int, Poke ()))
                           -> Peek a
                           -> Path Abs File
                           -> m a
                           -> m a
 versionedDecodeOrLoadImpl pokeFunc peekFunc fp mx = do
-    let fpt = T.pack (toFilePath fp)
+    let fpt = fromString (toFilePath fp)
     logDebug $ "Trying to decode " <> fpt
     mres <- versionedDecodeFileImpl peekFunc fp
     case mres of
@@ -75,20 +74,20 @@ versionedDecodeOrLoadImpl pokeFunc peekFunc fp mx = do
             storeEncodeFile pokeFunc peekFunc fp x
             return x
 
-versionedDecodeFileImpl :: (Store a, MonadUnliftIO m, MonadLogger m)
+versionedDecodeFileImpl :: (Store a, MonadUnliftIO m, MonadReader env m, HasCallStack, HasLogFunc env)
                         => Peek a
                         -> Path loc File
                         -> m (Maybe a)
 versionedDecodeFileImpl peekFunc fp = do
     mbs <- liftIO (Just <$> BS.readFile (toFilePath fp)) `catch` \(err :: IOException) -> do
-        logDebug ("Exception ignored when attempting to load " <> T.pack (toFilePath fp) <> ": " <> T.pack (show err))
+        logDebug ("Exception ignored when attempting to load " <> fromString (toFilePath fp) <> ": " <> displayShow err)
         return Nothing
     case mbs of
         Nothing -> return Nothing
         Just bs ->
             liftIO (Just <$> decodeIOWith peekFunc bs) `catch` \(err :: PeekException) -> do
-                 let fpt = T.pack (toFilePath fp)
-                 logDebug ("Error while decoding " <> fpt <> ": " <> T.pack (show err) <> " (this might not be an error, when switching between stack versions)")
+                 let fpt = fromString (toFilePath fp)
+                 logDebug ("Error while decoding " <> fpt <> ": " <> displayShow err <> " (this might not be an error, when switching between stack versions)")
                  return Nothing
 
 storeVersionConfig :: String -> String -> VersionConfig a
