@@ -20,7 +20,6 @@ module Stack.BuildPlan
     , gpdPackages
     , removeSrcPkgDefaultFlags
     , selectBestSnapshot
-    , getToolMap
     , showItems
     ) where
 
@@ -36,10 +35,8 @@ import qualified Data.Text as T
 import qualified Distribution.Package as C
 import           Distribution.PackageDescription (GenericPackageDescription,
                                                   flagDefault, flagManual,
-                                                  flagName, genPackageFlags,
-                                                  condExecutables)
+                                                  flagName, genPackageFlags)
 import qualified Distribution.PackageDescription as C
-import qualified Distribution.Types.UnqualComponentName as C
 import           Distribution.System (Platform)
 import           Distribution.Text (display)
 import qualified Distribution.Version as C
@@ -49,7 +46,6 @@ import           Stack.Package
 import           Stack.Snapshot
 import           Stack.Types.BuildPlan
 import           Stack.Types.FlagName
-import           Stack.Types.NamedComponent
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
@@ -145,47 +141,6 @@ instance Show BuildPlanException where
         T.unpack url ++
         ", because no 'compiler' or 'resolver' is specified."
 
--- | Map from tool name to package providing it. This accounts for
--- both snapshot and local packages (deps and project packages).
-getToolMap :: LoadedSnapshot
-           -> LocalPackages
-           -> Map ExeName (Set PackageName)
-getToolMap ls locals =
-
-    {- We no longer do this, following discussion at:
-
-        https://github.com/commercialhaskell/stack/issues/308#issuecomment-112076704
-
-    -- First grab all of the package names, for times where a build tool is
-    -- identified by package name
-    $ Map.fromList (map (packageNameByteString &&& Set.singleton) (Map.keys ps))
-    -}
-
-    Map.unionsWith Set.union $ concat
-        [ concatMap goSnap      $ Map.toList $ lsPackages ls
-        , concatMap goLocalProj $ Map.toList $ lpProject locals
-        , concatMap goLocalDep  $ Map.toList $ lpDependencies locals
-        ]
-  where
-    goSnap (pname, lpi) =
-        map (flip Map.singleton (Set.singleton pname))
-      $ Set.toList
-      $ lpiProvidedExes lpi
-
-    goLocalProj (pname, lpv) =
-        map (flip Map.singleton (Set.singleton pname))
-        [ExeName t | CExe t <- Set.toList (lpvComponents lpv)]
-
-    goLocalDep (pname, (gpd, _loc)) =
-        map (flip Map.singleton (Set.singleton pname))
-      $ gpdExes gpd
-
-    -- TODO consider doing buildable checking. Not a big deal though:
-    -- worse case scenario is we build an extra package that wasn't
-    -- strictly needed.
-    gpdExes :: GenericPackageDescription -> [ExeName]
-    gpdExes = map (ExeName . T.pack . C.unUnqualComponentName . fst) . condExecutables
-
 gpdPackages :: [GenericPackageDescription] -> Map PackageName Version
 gpdPackages gpds = Map.fromList $
             map (fromCabalIdent . C.package . C.packageDescription) gpds
