@@ -200,12 +200,12 @@ execWithOptionalContainer mprojectRoot getCmdArgs mbefore inner mafter mrelease
             throwIO OnlyOnHostException
         | inContainer ->
             liftIO (do inner
-                       exitSuccess)
+                       System.Exit.exitSuccess)
         | not (dockerEnable (configDocker config)) ->
             do fromMaybeAction mbefore
                liftIO inner
                fromMaybeAction mafter
-               liftIO exitSuccess
+               liftIO System.Exit.exitSuccess
         | otherwise ->
             do fromMaybeAction mrelease
                runContainerAndExit
@@ -243,8 +243,8 @@ runContainerAndExit getCmdArgs
      (env,isStdinTerminal,isStderrTerminal,homeDir) <- liftIO $
        (,,,)
        <$> getEnvironment
-       <*> hIsTerminalDevice stdin
-       <*> hIsTerminalDevice stderr
+       <*> hIsTerminalDeviceOrMinTTY stdin
+       <*> hIsTerminalDeviceOrMinTTY stderr
        <*> (parseAbsDir =<< getHomeDirectory)
      isStdoutTerminal <- view terminalL
      let dockerHost = lookup "DOCKER_HOST" env
@@ -376,9 +376,9 @@ runContainerAndExit getCmdArgs
 #endif
          )
      case e of
-       Left (ProcessExitedUnsuccessfully _ ec) -> liftIO (exitWith ec)
+       Left (ProcessExitedUnsuccessfully _ ec) -> liftIO (System.Exit.exitWith ec)
        Right () -> do after
-                      liftIO exitSuccess
+                      liftIO System.Exit.exitSuccess
   where
     -- This is using a hash of the Docker repository (without tag or digest) to ensure
     -- binaries/libraries aren't shared between Docker and host (or incompatible Docker images)
