@@ -1,10 +1,12 @@
+{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE DataKinds #-}
 module Stack.Options.ResolverParser where
 
-import           Data.Monoid.Extra
 import qualified Data.Text                         as T
 import           Options.Applicative
 import           Options.Applicative.Types         (readerAsk)
 import           Stack.Options.Utils
+import           Stack.Prelude
 import           Stack.Types.Compiler
 import           Stack.Types.Resolver
 
@@ -17,7 +19,7 @@ abstractResolverOptsParser hide =
          help "Override resolver in project file" <>
          hideMods hide)
 
-compilerOptsParser :: Bool -> Parser CompilerVersion
+compilerOptsParser :: Bool -> Parser (CompilerVersion 'CVWanted)
 compilerOptsParser hide =
     option readCompilerVersion
         (long "compiler" <>
@@ -25,7 +27,7 @@ compilerOptsParser hide =
          help "Use the specified compiler" <>
          hideMods hide)
 
-readCompilerVersion :: ReadM CompilerVersion
+readCompilerVersion :: ReadM (CompilerVersion 'CVWanted)
 readCompilerVersion = do
     s <- readerAsk
     case parseCompilerVersion (T.pack s) of
