@@ -20,19 +20,19 @@ import           Path.Extra
 import           Stack.Constants
 import           Stack.Constants.Config
 import           Stack.GhcPkg as GhcPkg
+import           Stack.PackageIndex (HasCabalLoader (..))
 import           Stack.Types.Config
 import           Stack.Types.Runner
 import qualified System.FilePath as FP
 import           System.IO (stderr)
-import           System.Process.Read (EnvOverride(eoPath))
+import           RIO.Process (HasProcessContext (..), exeSearchPathL)
 
 -- | Print out useful path information in a human-readable format (and
 -- support others later).
 path
-    :: (MonadUnliftIO m, MonadReader env m, HasEnvConfig env, MonadThrow m,
-        MonadLogger m)
+    :: HasEnvConfig env
     => [Text]
-    -> m ()
+    -> RIO env ()
 path keys =
     do -- We must use a BuildConfig from an EnvConfig to ensure that it contains the
        -- full environment info including GHC paths etc.
@@ -42,12 +42,11 @@ path keys =
        -- global GHC.
        -- It was set up in 'withBuildConfigAndLock -> withBuildConfigExt -> setupEnv'.
        -- So it's not the *minimal* override path.
-       menv <- getMinimalEnvOverride
        snap <- packageDatabaseDeps
        plocal <- packageDatabaseLocal
        extra <- packageDatabaseExtra
        whichCompiler <- view $ actualCompilerVersionL.whichCompilerL
-       global <- GhcPkg.getGlobalDB menv whichCompiler
+       global <- GhcPkg.getGlobalDB whichCompiler
        snaproot <- installationRootDeps
        localroot <- installationRootLocal
        toolsDir <- bindirCompilerTools
@@ -78,7 +77,6 @@ path keys =
                       path'
                           (PathInfo
                                bc
-                               menv
                                snap
                                plocal
                                global
@@ -103,7 +101,6 @@ pathParser =
 -- | Passed to all the path printers as a source of info.
 data PathInfo = PathInfo
     { piBuildConfig  :: BuildConfig
-    , piEnvOverride  :: EnvOverride
     , piSnapDb       :: Path Abs Dir
     , piLocalDb      :: Path Abs Dir
     , piGlobalDb     :: Path Abs Dir
@@ -122,6 +119,10 @@ instance HasLogFunc PathInfo where
 instance HasRunner PathInfo where
     runnerL = configL.runnerL
 instance HasConfig PathInfo
+instance HasCabalLoader PathInfo where
+    cabalLoaderL = configL.cabalLoaderL
+instance HasProcessContext PathInfo where
+    processContextL = configL.processContextL
 instance HasBuildConfig PathInfo where
     buildConfigL = lens piBuildConfig (\x y -> x { piBuildConfig = y })
                  . buildConfigL
@@ -148,7 +149,7 @@ paths =
       , view $ stackYamlL.to toFilePath.to T.pack)
     , ( "PATH environment variable"
       , "bin-path"
-      , T.pack . intercalate [FP.searchPathSeparator] . eoPath . piEnvOverride )
+      , T.pack . intercalate [FP.searchPathSeparator] . view exeSearchPathL)
     , ( "Install location for GHC and other core tools"
       , "programs"
       , view $ configL.to configLocalPrograms.to toFilePathNoTrailingSep.to T.pack)
