@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP                        #-}
 {-# LANGUAGE NoImplicitPrelude          #-}
 {-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
@@ -13,6 +14,10 @@ module Stack.Prelude
   , readProcessNull
   , withProcessContext
   , stripCR
+  , hIsTerminalDeviceOrMinTTY
+  , prompt
+  , promptPassword
+  , promptBool
   , module X
   ) where
 
@@ -28,17 +33,23 @@ import qualified Path.IO
 import qualified System.IO as IO
 import qualified System.Directory as Dir
 import qualified System.FilePath as FP
+import           System.IO.Echo (withoutInputEcho)
 import           System.IO.Error (isDoesNotExistError)
 
+#ifdef WINDOWS
+import           System.Win32 (isMinTTYHandle, withHandleToHANDLE)
+#endif
+
 import           Data.Conduit.Binary (sourceHandle, sinkHandle)
 import qualified Data.Conduit.Binary as CB
 import qualified Data.Conduit.List as CL
 import           Data.Conduit.Process.Typed (withLoggedProcess_, createSource)
-import           RIO.Process (HasProcessContext (..), ProcessContext, setStdin, closed, getStderr, getStdout, proc, withProcess_, setStdout, setStderr, ProcessConfig, readProcessStdout_, workingDirL)
+import           RIO.Process (HasProcessContext (..), ProcessContext, setStdin, closed, getStderr, getStdout, proc, withProcess_, setStdout, setStderr, ProcessConfig, readProcess_, workingDirL)
 import           Data.Store           as X (Store)
 import           Data.Text.Encoding (decodeUtf8With)
 import           Data.Text.Encoding.Error (lenientDecode)
 
+import qualified Data.Text.IO as T
 import qualified RIO.Text as T
 
 -- | Get a source for a file. Unlike @sourceFile@, doesn't require
@@ -86,7 +97,7 @@ withKeepSystemTempDir str inner = withRunInIO $ \run -> do
 --
 -- Throws a 'ReadProcessException' if unsuccessful in launching, or 'ProcessExitedUnsuccessfully' if the process itself fails.
 sinkProcessStderrStdout
-  :: forall e o env. (HasProcessContext env, HasLogFunc env)
+  :: forall e o env. (HasProcessContext env, HasLogFunc env, HasCallStack)
   => String -- ^ Command
   -> [String] -- ^ Command line arguments
   -> ConduitM ByteString Void (RIO env) e -- ^ Sink for stderr
@@ -108,7 +119,7 @@ sinkProcessStderrStdout name args sinkStderr sinkStdout =
 --
 -- Throws a 'ReadProcessException' if unsuccessful.
 sinkProcessStdout
-    :: (HasProcessContext env, HasLogFunc env)
+    :: (HasProcessContext env, HasLogFunc env, HasCallStack)
     => String -- ^ Command
     -> [String] -- ^ Command line arguments
     -> ConduitM ByteString Void (RIO env) a -- ^ Sink for stdout
@@ -132,13 +143,13 @@ logProcessStderrStdout pc = withLoggedProcess_ pc $ \p ->
 -- | Read from the process, ignoring any output.
 --
 -- Throws a 'ReadProcessException' exception if the process fails.
-readProcessNull :: (HasProcessContext env, HasLogFunc env)
+readProcessNull :: (HasProcessContext env, HasLogFunc env, HasCallStack)
                 => String -- ^ Command
                 -> [String] -- ^ Command line arguments
                 -> RIO env ()
 readProcessNull name args =
   -- We want the output to appear in any exceptions, so we capture and drop it
-  void $ proc name args readProcessStdout_
+  void $ proc name args readProcess_
 
 -- | Use the new 'ProcessContext', but retain the working directory
 -- from the parent environment.
@@ -151,3 +162,54 @@ withProcessContext pcNew inner = do
 -- | Remove a trailing carriage return if present
 stripCR :: Text -> Text
 stripCR = T.dropSuffix "\r"
+
+-- | hIsTerminaDevice does not recognise handles to mintty terminals as terminal
+-- devices, but isMinTTYHandle does.
+hIsTerminalDeviceOrMinTTY :: MonadIO m => Handle -> m Bool
+#ifdef WINDOWS
+hIsTerminalDeviceOrMinTTY h = do
+  isTD <- hIsTerminalDevice h
+  if isTD
+    then return True
+    else liftIO $ withHandleToHANDLE h isMinTTYHandle
+#else
+hIsTerminalDeviceOrMinTTY = hIsTerminalDevice
+#endif
+
+-- | Prompt the user by sending text to stdout, and taking a line of
+-- input from stdin.
+prompt :: MonadIO m => Text -> m Text
+prompt txt = liftIO $ do
+  T.putStr txt
+  hFlush stdout
+  T.getLine
+
+-- | Prompt the user by sending text to stdout, and collecting a line
+-- of input from stdin. While taking input from stdin, input echoing is
+-- disabled, to hide passwords.
+--
+-- Based on code from cabal-install, Distribution.Client.Upload
+promptPassword :: MonadIO m => Text -> m Text
+promptPassword txt = liftIO $ do
+  T.putStr txt
+  hFlush stdout
+  -- Save/restore the terminal echoing status (no echoing for entering
+  -- the password).
+  password <- withoutInputEcho T.getLine
+  -- Since the user's newline is not echoed, one needs to be inserted.
+  T.putStrLn ""
+  return password
+
+-- | Prompt the user by sending text to stdout, and collecting a line of
+-- input from stdin. If something other than "y" or "n" is entered, then
+-- print a message indicating that "y" or "n" is expected, and ask
+-- again.
+promptBool :: MonadIO m => Text -> m Bool
+promptBool txt = liftIO $ do
+  input <- prompt txt
+  case input of
+    "y" -> return True
+    "n" -> return False
+    _ -> do
+      T.putStrLn "Please press either 'y' or 'n', and then enter."
+      promptBool txt
