@@ -55,7 +55,7 @@ toolNameString (Tool ident) = packageNameString $ packageIdentifierName ident
 toolNameString ToolGhcjs{} = "ghcjs"
 
 parseToolText :: Text -> Maybe Tool
-parseToolText (parseCompilerVersion -> Just (cv@GhcjsVersion{})) = Just (ToolGhcjs cv)
+parseToolText (parseCompilerVersion -> Just cv@GhcjsVersion{}) = Just (ToolGhcjs cv)
 parseToolText (parsePackageIdentifierFromString . T.unpack -> Just pkgId) = Just (Tool pkgId)
 parseToolText _ = Nothing
 
@@ -88,6 +88,16 @@ listInstalled programsPath = do
         x <- T.stripSuffix ".installed" $ T.pack $ toFilePath $ filename fp
         parseToolText x
 
+-- | See https://github.com/commercialhaskell/stack/issues/4086.
+warnAboutGHCJS :: HasLogFunc env => RIO env ()
+warnAboutGHCJS =
+    logWarn $ "Building a GHCJS project. " <> fromString ghcjsWarning
+
+ghcjsWarning :: String
+ghcjsWarning = unwords
+     [ "Note that GHCJS support in Stack is EXPERIMENTAL"
+     ]
+
 getCompilerVersion
   :: (HasProcessContext env, HasLogFunc env)
   => WhichCompiler
@@ -96,17 +106,18 @@ getCompilerVersion wc =
     case wc of
         Ghc -> do
             logDebug "Asking GHC for its version"
-            bs <- proc "ghc" ["--numeric-version"] readProcessStdout_
+            bs <- fst <$> proc "ghc" ["--numeric-version"] readProcess_
             let (_, ghcVersion) = versionFromEnd $ BL.toStrict bs
             x <- GhcVersion <$> parseVersion (T.decodeUtf8 ghcVersion)
             logDebug $ "GHC version is: " <> display x
             return x
         Ghcjs -> do
+            warnAboutGHCJS
             logDebug "Asking GHCJS for its version"
             -- Output looks like
             --
             -- The Glorious Glasgow Haskell Compilation System for JavaScript, version 0.1.0 (GHC 7.10.2)
-            bs <- proc "ghcjs" ["--version"] readProcessStdout_
+            bs <- fst <$> proc "ghcjs" ["--version"] readProcess_
             let (rest, ghcVersion) = T.decodeUtf8 <$> versionFromEnd (BL.toStrict bs)
                 (_, ghcjsVersion) = T.decodeUtf8 <$> versionFromEnd rest
             GhcjsVersion <$> parseVersion ghcjsVersion <*> parseVersion ghcVersion
