@@ -1,4 +1,5 @@
 {-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE CPP                        #-}
 {-# LANGUAGE DataKinds                  #-}
 {-# LANGUAGE DeriveDataTypeable         #-}
 {-# LANGUAGE DeriveGeneric              #-}
@@ -671,11 +672,13 @@ instance Store ConfigureOpts
 instance NFData ConfigureOpts
 
 -- | Information on a compiled package: the library conf file (if relevant),
--- and all of the executable paths.
+-- the sublibraries (if present) and all of the executable paths.
 data PrecompiledCache = PrecompiledCache
     -- Use FilePath instead of Path Abs File for Binary instances
     { pcLibrary :: !(Maybe FilePath)
     -- ^ .conf file inside the package database
+    , pcSubLibs :: ![FilePath]
+    -- ^ .conf file inside the package database, for each of the sublibraries
     , pcExes    :: ![FilePath]
     -- ^ Full paths to executables
     }
@@ -684,4 +687,8 @@ instance Store PrecompiledCache
 instance NFData PrecompiledCache
 
 precompiledCacheVC :: VersionConfig PrecompiledCache
-precompiledCacheVC = storeVersionConfig "precompiled-v1" "eMzSOwaHJMamA5iNKs1A025frlQ="
+#if MIN_VERSION_template_haskell(2,14,0)
+precompiledCacheVC = storeVersionConfig "precompiled-v2" "1Q08F5_iKDGDMPCuBG0-Av9nEKk="
+#else
+precompiledCacheVC = storeVersionConfig "precompiled-v2" "55vMMtbIlS4UukKnSmjs1SrI01o="
+#endif
