@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 module Stack.Types.CompilerBuild
   (CompilerBuild(..)
   ,compilerBuildName
@@ -5,7 +6,7 @@ module Stack.Types.CompilerBuild
   ,parseCompilerBuild
   ) where
 
-import           Control.Monad.Catch (MonadThrow)
+import           Stack.Prelude
 import           Data.Aeson.Extended (FromJSON, parseJSON, withText)
 import           Data.Text as T
 
