@@ -81,6 +81,8 @@ instance Show Version where
   show (Version v) =
     intercalate "."
                 (map show (V.toList v))
+instance Display Version where
+  display = display . versionText
 
 instance ToJSON Version where
   toJSON = toJSON . versionText
@@ -99,10 +101,13 @@ newtype IntersectingVersionRange =
     IntersectingVersionRange { getIntersectingVersionRange :: Cabal.VersionRange }
     deriving Show
 
+instance Semigroup IntersectingVersionRange where
+    IntersectingVersionRange l <> IntersectingVersionRange r =
+        IntersectingVersionRange (l `Cabal.intersectVersionRanges` r)
+
 instance Monoid IntersectingVersionRange where
     mempty = IntersectingVersionRange Cabal.anyVersion
-    mappend (IntersectingVersionRange l) (IntersectingVersionRange r) =
-        IntersectingVersionRange (l `Cabal.intersectVersionRanges` r)
+    mappend = (<>)
 
 -- | Attoparsec parser for a package version.
 versionParser :: Parser Version
