@@ -25,18 +25,12 @@ import qualified Data.ByteString.Lazy                  as L
 import qualified Data.Conduit.Binary                   as CB
 import qualified Data.Text                             as T
 import           Data.Text.Encoding                    (encodeUtf8)
-import qualified Data.Text.IO                          as TIO
-import           Network.HTTP.Client                   (Response,
-                                                        RequestBody(RequestBodyLBS),
-                                                        Request)
-import           Network.HTTP.StackClient              (withResponse, httpNoBody)
-import           Network.HTTP.Simple                   (getResponseStatusCode,
+import           Network.HTTP.StackClient              (Request, RequestBody(RequestBodyLBS), Response, withResponse, httpNoBody, getGlobalManager, getResponseStatusCode,
                                                         getResponseBody,
                                                         setRequestHeader,
-                                                        parseRequest)
-import           Network.HTTP.Client.MultipartFormData (formDataBody, partFileRequestBody,
-                                                        partBS, partLBS)
-import           Network.HTTP.Client.TLS               (getGlobalManager,
+                                                        parseRequest,
+                                                        formDataBody, partFileRequestBody,
+                                                        partBS, partLBS,
                                                         applyDigestAuth,
                                                         displayDigestAuthException)
 import           Stack.Types.Config
@@ -46,8 +40,7 @@ import           Stack.Types.PackageName               (packageNameString)
 import           System.Directory                      (createDirectoryIfMissing,
                                                         removeFile)
 import           System.FilePath                       ((</>), takeFileName)
-import           System.IO                             (stdout, putStrLn, putStr, getLine, print) -- TODO remove putStrLn, use logInfo
-import           System.IO.Echo                        (withoutInputEcho)
+import           System.IO                             (stdout, putStrLn, putStr, print) -- TODO remove putStrLn, use logInfo
 
 -- | Username and password to log into Hackage.
 --
@@ -87,10 +80,8 @@ loadCreds config = do
       return $ mkCreds fp
   where
     fromPrompt fp = do
-      putStr "Hackage username: "
-      hFlush stdout
-      username <- TIO.getLine
-      password <- promptPassword
+      username <- prompt "Hackage username: "
+      password <- promptPassword "Hackage password: "
       let hc = HackageCreds
             { hcUsername = username
             , hcPassword = password
@@ -98,46 +89,22 @@ loadCreds config = do
             }
 
       when (configSaveHackageCreds config) $ do
-        let prompt = "Save hackage credentials to file at " ++ fp ++ " [y/n]? "
-        putStr prompt
-        input <- loopPrompt prompt
+        shouldSave <- promptBool $ T.pack $
+          "Save hackage credentials to file at " ++ fp ++ " [y/n]? "
         putStrLn "NOTE: Avoid this prompt in the future by using: save-hackage-creds: false"
-        hFlush stdout
-        case input of
-          "y" -> do
-            L.writeFile fp (encode hc)
-            putStrLn "Saved!"
-            hFlush stdout
-          _ -> return ()
+        when shouldSave $ do
+          L.writeFile fp (encode hc)
+          putStrLn "Saved!"
+          hFlush stdout
 
       return hc
 
-    loopPrompt :: String -> IO String
-    loopPrompt p = do
-      input <- TIO.getLine
-      case input of
-        "y" -> return "y"
-        "n" -> return "n"
-        _   -> do
-          putStr p
-          loopPrompt p
-
 credsFile :: Config -> IO FilePath
 credsFile config = do
     let dir = toFilePath (view stackRootL config) </> "upload"
     createDirectoryIfMissing True dir
     return $ dir </> "credentials.json"
 
--- | Lifted from cabal-install, Distribution.Client.Upload
-promptPassword :: IO Text
-promptPassword = do
-  putStr "Hackage password: "
-  hFlush stdout
-  -- save/restore the terminal echoing status (no echoing for entering the password)
-  passwd <- withoutInputEcho $ fmap T.pack getLine
-  putStrLn ""
-  return passwd
-
 applyCreds :: HackageCreds -> Request -> IO Request
 applyCreds creds req0 = do
   manager <- getGlobalManager
@@ -159,13 +126,14 @@ applyCreds creds req0 = do
 -- sending a file like 'upload', this sends a lazy bytestring.
 --
 -- Since 0.1.2.1
-uploadBytes :: HackageCreds
+uploadBytes :: String -- ^ Hackage base URL
+            -> HackageCreds
             -> String -- ^ tar file name
             -> L.ByteString -- ^ tar file contents
             -> IO ()
-uploadBytes creds tarName bytes = do
+uploadBytes baseUrl creds tarName bytes = do
     let req1 = setRequestHeader "Accept" ["text/plain"]
-               "https://hackage.haskell.org/packages/"
+               (fromString $ baseUrl <> "packages/")
         formData = [partFileRequestBody "package" tarName (RequestBodyLBS bytes)]
     req2 <- formDataBody formData req1
     req3 <- applyCreds creds req2
@@ -199,16 +167,21 @@ printBody res = runConduit $ getResponseBody res .| CB.sinkHandle stdout
 -- | Upload a single tarball with the given @Uploader@.
 --
 -- Since 0.1.0.0
-upload :: HackageCreds -> FilePath -> IO ()
-upload creds fp = uploadBytes creds (takeFileName fp) =<< L.readFile fp
+upload :: String -- ^ Hackage base URL
+       -> HackageCreds
+       -> FilePath
+       -> IO ()
+upload baseUrl creds fp = uploadBytes baseUrl creds (takeFileName fp) =<< L.readFile fp
 
-uploadRevision :: HackageCreds
+uploadRevision :: String -- ^ Hackage base URL
+               -> HackageCreds
                -> PackageIdentifier
                -> L.ByteString
                -> IO ()
-uploadRevision creds ident cabalFile = do
+uploadRevision baseUrl creds ident cabalFile = do
   req0 <- parseRequest $ concat
-    [ "https://hackage.haskell.org/package/"
+    [ baseUrl
+    , "package/"
     , packageIdentifierString ident
     , "/"
     , packageNameString $ packageIdentifierName ident
