@@ -125,6 +125,24 @@ stackErrStderr args check = do
         then error "Stack process succeeded, but it shouldn't"
         else check err
 
+stackStdout :: [String] -> IO (ExitCode, String)
+stackStdout args = do
+    stackExe' <- stackExe
+    logInfo $ "Running: " ++ stackExe' ++ " " ++ unwords (map showProcessArgDebug args)
+    (ec, out, err) <- readProcessWithExitCode stackExe' args ""
+    putStr out
+    hPutStr stderr err
+    return (ec, out)
+
+-- | Run stack with arguments and apply a check to the resulting
+-- stdout output if the process succeeded.
+stackCheckStdout :: [String] -> (String -> IO ()) -> IO ()
+stackCheckStdout args check = do
+    (ec, out) <- stackStdout args
+    if ec /= ExitSuccess
+        then error $ "Exited with exit code: " ++ show ec
+        else check out
+
 doesNotExist :: FilePath -> IO ()
 doesNotExist fp = do
     logInfo $ "doesNotExist " ++ fp
@@ -208,7 +226,7 @@ isMacOSX = os == "darwin"
 -- the main @stack.yaml@.
 --
 defaultResolverArg :: String
-defaultResolverArg = "--resolver=lts-11.6"
+defaultResolverArg = "--resolver=lts-11.22"
 
 -- | Remove a file and ignore any warnings about missing files.
 removeFileIgnore :: FilePath -> IO ()
@@ -216,3 +234,10 @@ removeFileIgnore fp = removeFile fp `catch` \e ->
   if isDoesNotExistError e
     then return ()
     else throwIO e
+
+-- | Remove a directory tree and ignore any warnings about missing files.
+removeDirectoryRecursiveIgnore :: FilePath -> IO ()
+removeDirectoryRecursiveIgnore fp = removeDirectoryRecursive fp `catch` \e ->
+  if isDoesNotExistError e
+    then return ()
+    else throwIO e
